/* -------------------------------------------------------------------------- 
 * Copyright (c) 2019 Arm Limited (or its affiliates). All 
 * rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *
 * $Date:        17. October 2019
 * $Revision:    V1.0
 *
 * Project:      USB1 common (Device and Host) module for 
 *               NXP i.MX RT 1060 Series
 * -------------------------------------------------------------------------- */

/* History:
 *  Version 1.0
 *    Initial release
 */

/*! \cond */

#include "cmsis_compiler.h"

#include "Driver_USB.h"

#include "RTE_Components.h"

volatile uint8_t USB1_role  = ARM_USB_ROLE_NONE;
volatile uint8_t USB1_state = 0U;

__WEAK void USBH1_IRQ (void) {};
__WEAK void USBD1_IRQ (void) {};

extern void USBH1_IRQ (void);
extern void USBD1_IRQ (void);


// Common IRQ Routine **********************************************************

/**
  \fn          void USB_OTG1_IRQHandler (void)
  \brief       USB1 Interrupt Routine (IRQ).
*/
void USB_OTG1_IRQHandler (void) {
  switch (USB1_role) {
    case ARM_USB_ROLE_HOST:
      USBH1_IRQ ();
      break;
    case ARM_USB_ROLE_DEVICE:
      USBD1_IRQ ();
      break;
    default:
      break;
  }
}

/*! \endcond */
