/*------------------------------------------------------------------------------
 * Name:    lpuart1.c
 * Purpose: fsl_lpuart_cmsis driver helper file
 *----------------------------------------------------------------------------*/

/*! \page evk_imxrt1060_uart1 CMSIS-Driver for UART1 Interface

The CMSIS-Driver for UART Interface is part of NXP DFP, below is an explanation 
on how UART1 driver can be used with MDK.

To use the CMSIS-Driver UART1, do the following:
- In the Manage Run-Time Environment window, enable <b>CMSIS Driver:USART (API):lpuart_cmsis</b>
- Copy <tt>%lpuart1.c</tt> file form <tt>\ref directory_spec "\<PACK_root\>"\\Templates</tt> to <tt>\ref directory_spec "\<project_dir\>"</tt> and add it to the project
- Copy <tt>RTE_Device.h</tt> file form <tt>\ref directory_spec "\<PACK_root\>"\\CMSIS\\Driver\\Config</tt> to 
  <tt>\ref directory_spec "\<project_dir\>"\\RTE\\Device\\\ref directory_spec "<selected_device\>"</tt>
- In project open <b>Device:RTE_Device.h</b>, you can use Configuration Wizard view, and enable <b>UART:UART1 [Driver_USART1]</b>.
  Optionally enable DMA for Tx and Rx Channels.
- Create \c LPUART1_InitPins and \c LPUART1_DeinitPins functions (refer to the pin configuration below)

\note
UART1 is routed through Virtual COM port on the USB.
You need to install the Mbed Windows serial driver to be able to communicate via UART1. Download and install the driver
from here:
<a target="_blank" href="https://os.mbed.com/docs/latest/tutorials/windows-serial-driver.html">Windows serial driver</a>.

<b>CMSIS-Driver for UART1 Interface Setup</b>

The CMSIS-Driver UART1 Interface requires:
  - UART1 Pins
  - UART1 Clock
 
Valid pin settings for <b>UART1</b> peripheral on <b>EVK-IMXRT1060</b> evaluation board are listed in the table below:
 
|  #  | Peripheral | Signal     | Route to      | Direction     |
|:----|:-----------|:-----------|:--------------|:--------------|
| L14 | LPUART1    | RX         | GPIO_AD_B0_13 | Input         |
| K14 | LPUART1    | TX         | GPIO_AD_B0_12 | Output        |
 
The rest of the settings can be at the default (reset) value.

For other boards or custom hardware, refer to the hardware schematics to reflect correct setup values.
 
In the \ref config_pinclock "MCUXpresso Config Tools", make sure that the following pin and clock settings are made (enter
the values that are shown in <em>italics</em>):
 
-# Select \b Pins -> \b Functional \b Groups main menu and create functions <em>LPUART1_InitPins</em> and <em>LPUART1_DeinitPins</em>
-# Under the \b Functional \b Group entry in the toolbar, select <b>LPUART1_InitPins</b>
-# Modify Routed Pins configuration to match the settings listed in the table above
-# Under the \b Functional \b Group entry in the toolbar, select <b>LPUART1_DeinitPins</b>
-# Modify Routed Pins configuration to match the settings required after UART pin deinitialization
-# Go to <b>Tools - Clocks</b>
-# Go to <b>Views - Clocks Diagram</b> and check that \b UART clock is set to <em>80 MHz</em>
-# Click on <b>Update Project</b> button to update source files
*/

/*! \cond */

#include "board.h"

uint32_t LPUART1_GetFreq(void) {
  return BOARD_DebugConsoleSrcFreq();
}

/*! \endcond */
